# coding: utf-8
from django.forms import model_to_dict
from django.shortcuts import render
from django.db.models import Sum, Avg
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.request import Request
from rest_framework.response import Response

from before_request import before_request
from inspect_report.agency import get_team_names
from inspect_report.dbhelper import db_static_score, db_seat_rule, db_seat_check, db_seat_area, db_static_rule, \
    set_city_country_by_team_name, db_city_score
from inspect_report.models import RulesStat, SeatStat, ScoreStat, Round, Seat, Tasks
import json
from django.views.decorators.clickjacking import xframe_options_exempt
import logging

from inspect_report.pagehelper import page_with_seat_name
from inspect_report.utils.report_utils import delete_stat_by_task, re_stat_by_task

logger = logging.getLogger(__name__)


class TasksApi(viewsets.ViewSet):
    authentication_classes = ()
    permission_classes = ()

    # permission_classes = (permissions.ValidateToken, )

    # @action(['get'], detail=False)
    # def obtain(self, req: Request):
    #     """
    #     查询任务列表
    #     :param req:
    #     :return:
    #     """
    #     tasks = Tasks.objects.all().order_by('-createdAt').values('id', 'name')
    #     # return render(req, 'inspect/index.html', {'tasks': tasks})
    #     return Response({'code': 0, 'msg': 'success', 'data': tasks})

    @action(['get'], detail=False)
    def stat_report(self, req: Request):
        task_id = req.GET.get('task_id')
        if not task_id:
            logger.error('[re_stat_report_data]---invalid params')
            return Response({'code': -1, 'msg': 'invalid params'})
        try:
            task = Tasks.objects.get(pk=task_id)
        except Exception as e:
            print(e)
            logger.error('[re_stat_report_data]---no task exists')
            return Response({'code': -2, 'msg': 'no task exists'})
        # 删除与task_id相关的统计
        delete_stat_by_task(task)
        # 重新统计
        re_stat_by_task(model_to_dict(task))
        return Response({'code': 0, 'msg': 're-stat success'})

    @action(['get'], detail=False)
    @before_request
    def seat(self, req: Request):
        """
        获取坐席列表
        :param req:
        :return:
        """
        city_id = req.GET.get('city', None)
        country_id = req.GET.get('country', None)
        team_id = req.GET.get('team', None)
        session_condition = req.data.get('date_condition', {})
        team_names = get_team_names(city_id, country_id, team_id)

        session_condition['task__in'] = team_names
        seat_ids = list(set(SeatStat.objects.filter(**session_condition).values_list('agentName', flat=True)))
        seat_id_names = Seat.objects.filter(code__in=seat_ids).values('code', 'name')
        seat_ids_with_name = [x['code'] for x in seat_id_names]
        extra = [{'code': seat_id, 'name': seat_id} for seat_id in seat_ids if seat_id not in seat_ids_with_name]
        return Response({'code': 0, 'msg': 'success', 'data': list(seat_id_names) + extra})

    # @action(['post'], detail=False)
    # def rule(self, req: Request):
    #     """
    #     获取违规项分析
    #     :param req:
    #     :return:
    #     """
    #     task_id = req.data.get('task', '')
    #     agent_name = req.data.get('agentName', '')
    #     start_date = req.data.get('start_date', (datetime.now() + timedelta(days=-1)).strftime('%Y-%m-%d'))
    #     end_date = req.data.get('end_date', datetime.now().strftime('%Y-%m-%d'))
    #     q_start_date = datetime.strptime(start_date, '%Y-%m-%d').date()
    #     q_end_date = (datetime.strptime(end_date, '%Y-%m-%d') + + timedelta(days=1)).date()
    #     task_condition = {'create_date__gte': q_start_date, 'create_date__lt': q_end_date}
    #     data_sort = []
    #     return_data = {}
    #     if task_id:
    #         task_condition['task'] = task_id
    #     if agent_name:
    #         task_condition['agentName'] = agent_name
    #     # tasks = RulesStat.objects.filter(**task_condition).values('rule', 'sessionCollectionId', 'rule_num')
    #     tasks = RulesStat.objects.all().filter(**task_condition) \
    #         .extra(select={'rule': "rule", 'sessionCollectionId': 'sessionCollectionId'}) \
    #         .values('rule', 'sessionCollectionId').annotate(rule_num=Count('id')).order_by('-rule_num')
    #     for t in tasks:
    #         name = t['rule']
    #         if name in return_data.keys():
    #             return_data[name]['count'] += int(t['rule_num'])
    #         else:
    #             return_data[name] = {'count': int(t['rule_num']), 'tables': []}
    #         if t['sessionCollectionId'] not in return_data[name]['tables']:
    #             return_data[name]['tables'].append(t['sessionCollectionId'])
    #
    #     for k, v in return_data.items():
    #         v['rule'] = k
    #         data_sort.append(v)
    #     # data_sort = sorted(data_sort, key=lambda x: x['count'], reverse=True)
    #     return Response({'code': 0, 'msg': 'success', 'data': data_sort})

    # @action(['post'], detail=False)
    # def seat_session(self, req: Request):
    #     """
    #     坐席违规分析-按照会话
    #     :param req:
    #     :return:
    #     """
    #     task_id = req.data.get('task', '')
    #     num = req.data.get('num', '10')
    #     start_date = req.data.get('start_date', (datetime.now() + timedelta(days=-1)).strftime('%Y-%m-%d'))
    #     end_date = req.data.get('end_date', datetime.now().strftime('%Y-%m-%d'))
    #     task_condition = {'hasCheck': 1, 'createdAt__gte': start_date, 'createdAt__lt': end_date + ' 23:59:59'}
    #     if task_id:
    #         task_condition['name__startswith'] = task_id
    #     tasks = Tasks.objects.filter(**task_condition).values('id', 'sessionCollectionId')
    #     task_dict = {}
    #     for t in tasks:
    #         if t['sessionCollectionId'] in task_dict.keys():
    #             task_dict[t['sessionCollectionId']].append(t['id'])
    #         else:
    #             task_dict[t['sessionCollectionId']] = [t['id'], ]
    #     return_data = {}
    #     for k, v in task_dict.items():
    #         table_name = TABLE_PRE + k
    #         tn = CheckSession.set_table(table_name)
    #         session_condition = {'taskId__in': v}
    #         checks = tn.objects.filter(**session_condition).values('agentName')
    #         for check in checks:
    #             agent_name = check['agentName']
    #             if agent_name:
    #                 if agent_name in return_data.keys():
    #                     return_data[agent_name] += 1
    #                 else:
    #                     return_data[agent_name] = 1
    #     data_max = sorted(return_data.items(), key=lambda item: item[1], reverse=False)
    #     data_max = data_max[-int(num):]
    #     data_max_return = {}
    #     for dm in data_max:
    #         data_max_return[dm[0]] = dm[1]
    #     return Response({'code': 0, 'msg': 'success', 'data': data_max_return})

    # @action(['post'], detail=False)
    # def seat_score(self, req: Request):
    #     """
    #     坐席违规分析-按照得分
    #     :param req:
    #     :return:
    #     """
    #     task_id = req.data.get('task', '')
    #     num = req.data.get('num', '10')
    #     start_date = req.data.get('start_date', (datetime.now() + timedelta(days=-1)).strftime('%Y-%m-%d'))
    #     end_date = req.data.get('end_date', datetime.now().strftime('%Y-%m-%d'))
    #     task_condition = {'hasCheck': 1, 'createdAt__gte': start_date, 'createdAt__lt': end_date + ' 23:59:59'}
    #     if task_id:
    #         task_condition['name__startswith'] = task_id
    #     tasks = Tasks.objects.filter(**task_condition).values('id', 'sessionCollectionId')
    #     task_dict = {}
    #     for t in tasks:
    #         if t['sessionCollectionId'] in task_dict.keys():
    #             task_dict[t['sessionCollectionId']].append(t['id'])
    #         else:
    #             task_dict[t['sessionCollectionId']] = [t['id'], ]
    #     return_data = {}
    #     for k, v in task_dict.items():
    #         table_name = TABLE_PRE + k
    #         tn = CheckSession.set_table(table_name)
    #         session_condition = {'taskId__in': v}
    #         checks = tn.objects.filter(**session_condition).values('agentName', 'score')
    #         for check in checks:
    #             agent_name = check['agentName']
    #             if agent_name:
    #                 if agent_name in return_data.keys():
    #                     return_data[agent_name] += check['score']
    #                 else:
    #                     return_data[agent_name] = check['score']
    #     data_max = sorted(return_data.items(), key=lambda item: item[1], reverse=False)
    #     data_max = data_max[:int(num)]
    #     data_max_return = {}
    #     for dm in data_max:
    #         data_max_return[dm[0]] = dm[1]
    #     return Response({'code': 0, 'msg': 'success', 'data': data_max_return})

    @action(['post'], detail=False)
    @before_request
    def seat_rule(self, req: Request):
        """
        坐席违规分析-按照得分
        :param req:
        :return:
        """
        total_count, data_sort = db_seat_rule(req, if_page=True)
        return Response({'code': 0, 'msg': 'success', 'count': total_count, 'data': data_sort})

    @action(['post'], detail=False)
    @before_request
    def static(self, req: Request):
        """
        获取违规项分析
        :param req:
        :return:
        """
        city_id = req.data.get('city', None)
        country_id = req.data.get('country', None)
        session_condition = req.data.get('date_condition', {})

        team_names = get_team_names(city_id, country_id)
        session_condition['task__in'] = team_names

        seat_data = SeatStat.objects.filter(**session_condition).aggregate(
            validate_session_sum=Sum('validate_session'), total_session_sum=Sum('total_session'))
        avg_score = ScoreStat.objects.filter(**session_condition).aggregate(score_avg=Round(Avg('score'), 2))
        return_data = {'validate_session': seat_data['validate_session_sum'],
                       'total_session': seat_data['total_session_sum']}
        if seat_data['total_session_sum']:
            return_data['ratio'] = round(seat_data['validate_session_sum'] / seat_data['total_session_sum'], 2)
            return_data['avg_score'] = avg_score['score_avg']
        else:
            return_data['ratio'] = 0.0
            return_data['avg_score'] = 0.0
        return Response({'code': 0, 'msg': 'success', 'data': return_data})

    # @action(['post'], detail=False)
    # def static_date(self, req: Request):
    #     """
    #     获取违规项分析
    #     :param req:
    #     :return:
    #     """
    #     task_id = req.data.get('task', '')
    #     start_date = req.data.get('start_date', (datetime.now() + timedelta(days=-6)).strftime('%Y-%m-%d'))
    #     end_date = req.data.get('end_date', datetime.now().strftime('%Y-%m-%d'))
    #     task_condition = {'hasCheck': 1, 'createdAt__gte': start_date, 'createdAt__lt': end_date + ' 23:59:59'}
    #     if task_id:
    #         task_condition['name__startswith'] = task_id
    #     tasks = Tasks.objects.filter(**task_condition).values('id', 'sessionCollectionId')
    #     task_dict = {}
    #     for t in tasks:
    #         if t['sessionCollectionId'] in task_dict.keys():
    #             task_dict[t['sessionCollectionId']].append(t['id'])
    #         else:
    #             task_dict[t['sessionCollectionId']] = [t['id'], ]
    #     return_data = {}
    #     for k, v in task_dict.items():
    #         table_name = TABLE_PRE + k
    #         tn = CheckSession.set_table(table_name)
    #         session_condition = {'taskId__in': v}
    #         checks = tn.objects.filter(**session_condition).values('violationRuleCount', 'createdAt')
    #         for check in checks:
    #             date = check['createdAt'].strftime('%Y-%m-%d')
    #             if date in return_data.keys():
    #                 return_data[date]['total_session'] += 1
    #                 if check['violationRuleCount'] > 0:
    #                     return_data[date]['validate_session'] += 1
    #             else:
    #                 if check['violationRuleCount'] > 0:
    #                     return_data[date] = {'total_session': 1, 'validate_session': 1}
    #                 else:
    #                     return_data[date] = {'total_session': 1, 'validate_session': 0}
    #     data_sort = []
    #     for k, v in return_data.items():
    #         # return_data[k]['ratio'] = round(v['validate_session']/v['total_session'], 2)
    #         if v['total_session']:
    #             v['ratio'] = round(v['validate_session'] / v['total_session'], 2)
    #         else:
    #             v['ratio'] = 0
    #         v['date'] = k
    #         data_sort.append(v)
    #     data_sort = sorted(data_sort, key=lambda x: x['date'], reverse=True)
    #     return Response({'code': 0, 'msg': 'success', 'data': data_sort})

    @action(['post'], detail=False)
    @before_request
    def seat_check(self, req: Request):
        """
        坐席绩效考核汇总
        :param req:
        :return:
        """
        total_count, return_data = db_seat_check(req, if_page=True)
        return Response({'code': 0, 'msg': 'success', 'count': total_count, 'data': return_data})

    @action(['post'], detail=False)
    @before_request
    def seat_area(self, req: Request):
        """
        地市坐席成绩分析
        :param req:
        :return:
        """
        page = req.data.get('page', '1')
        page_size = req.data.get('page_size', '10')
        all_scores = db_seat_area(req)
        total_count, scores = page_with_seat_name(all_scores, page, page_size, agent=True)

        return Response({'code': 0, 'msg': 'success', 'count': total_count, 'data': scores.object_list})

    @action(['post'], detail=False)
    @before_request
    def static_rule(self, req: Request):
        """
        首页概述-违规项统计
        :param req:
        :return:
        """

        data_sort = db_static_rule(req)
        # data_sort = sorted(data_sort, key=lambda x: x['count'], reverse=True)
        return Response({'code': 0, 'msg': 'success', 'data': data_sort})

    @action(['post'], detail=False)
    @before_request
    def rule_detail(self, req: Request):
        """
        首页概述-违规项统计
        :param req:
        :return:
        """
        city_id = req.data.get('city', None)
        country_id = req.data.get('country', None)
        team_id = req.data.get('team', None)
        tables = req.data.get('tables', '')
        rule_name = req.data.get('rule', '')
        page = req.data.get('page', '1')
        page_size = req.data.get('page_size', '10')
        if not isinstance(tables, list):
            table_list = json.loads(tables)
        else:
            table_list = tables
        session_condition = req.data.get('date_condition', {})
        session_condition['rule'] = rule_name
        session_condition['sessionCollectionId__in'] = table_list
        session_condition['task__in'] = get_team_names(city_id, country_id, team_id)
        all_rules = RulesStat.objects.filter(**session_condition).values('agentName', 'customName', 'remainTime',
                                                                         'score',
                                                                         'sessionCollectionId', 'sessionId', 'taskId')
        total_count, rules = page_with_seat_name(all_rules, page, page_size, agent=True)

        return Response({'code': 0, 'msg': 'success', 'count': total_count, 'data': rules.object_list})

    @action(['post'], detail=False)
    @before_request
    def static_score(self, req: Request):
        """
        概览-团队得分统计
        :param req:
        :return:
        """
        page = req.data.get('page', '1')
        page_size = req.data.get('page_size', '10')

        all_scores = db_static_score(req)

        total_count, scores = page_with_seat_name(all_scores, page, page_size)

        scores = set_city_country_by_team_name(scores.object_list, 'area')

        return Response(
            {'code': 0, 'msg': 'success', 'count': total_count, 'data': scores})

    @action(['post'], detail=False)
    @before_request
    def city_score(self, req: Request):
        """
        地市质检成绩汇总分析
        :param req:
        :return:
        """
        city_score_list = db_city_score(req)
        return Response({'code': 0, 'msg': 'success', 'data': city_score_list})

    @action(['post'], detail=False)
    @before_request
    def score_detail(self, req: Request):
        """
        坐席得分列表
        :param req:
        :return:
        """
        team = req.data.get('team', '')
        agent_name = req.data.get('agentName', '')
        page = req.data.get('page', '1')
        page_size = req.data.get('page_size', '10')
        session_condition = req.data.get('date_condition', {})

        if team:
            session_condition['task'] = team

        if agent_name:
            session_condition['agentName'] = agent_name

        all_rules = RulesStat.objects.filter(**session_condition) \
            .extra(select={'sessionCollectionId': 'sessionCollectionId'}) \
            .values('agentName', 'customName', 'remainTime', 'score', 'sessionCollectionId', 'sessionId', 'taskId') \
            .distinct()

        total_count, rules = page_with_seat_name(all_rules, page, page_size, agent=True)

        return Response({'code': 0, 'msg': 'success', 'count': total_count, 'data': rules.object_list})


@xframe_options_exempt
def rule(request):
    return render(request, 'inspect/rule.html')


@xframe_options_exempt
def seat(request):
    return render(request, 'inspect/seat.html')


@xframe_options_exempt
def group(request):
    return render(request, 'inspect/group.html')


@xframe_options_exempt
def score(request):
    return render(request, 'inspect/score.html')


@xframe_options_exempt
def check(request):
    return render(request, 'inspect/seat_check.html')


@xframe_options_exempt
def city_score(request):
    return render(request, 'inspect/city_score.html')
